/*    
** mgpaint: the mgpaint for MiniGUI.
** Copyright (C) 2002~2007  Feynman Software

** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License along
** with this program; if not, write to the Free Software Foundation, Inc.,
** 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#ifndef _OPERATION_H_
#define _OPERATION_H_

#include <sys/types.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#ifdef WIN32
#include <dirent_win32.h>
#else
#include <dirent.h>
#endif
#include <minigui/common.h>
#include <minigui/minigui.h>
#include <minigui/gdi.h>
#include <minigui/window.h>
#include <minigui/control.h>
#include <minigui/colordlg.h>
#include "resource.h"

extern HWND hModesel;
extern HWND hCanvas;
extern HWND hCanvasFrm;
extern HDC  hCanvDC;
extern HDC  hCanvDCBak;
extern HDC  hCanvDCOper;
extern int fgclr;
extern int bgclr;
extern POINT pt[];
extern int  ptnum;
extern RECT bakrc;
extern int  opmode;
extern int wid, hei;
extern float sc[];
extern float mulview;
extern STIPPLE my_stipple;
extern STIPPLE my_stipple1;
extern BLOCKHEAP my_cliprc_heap;
extern int      operate ;
extern HWND      lastcapwnd;
extern int mleft, mtop, mright, mbottom;
extern unsigned char binit;
extern RECT selrc;

#define fgr     GetRValue(fgclr)
#define fgg     GetGValue(fgclr)
#define fgb     GetBValue(fgclr)
#define bgr     GetRValue(bgclr)
#define bgg     GetGValue(bgclr)
#define bgb     GetBValue(bgclr)

#define LIMIT(a, b, c ) do {a=((a<b)?b:a);a=(a>c)?c:a;} while (0)

void SprayTo (HDC dc, int x, int y, int w, int h, int r, int g , int b, double p , int s );

struct tag_operate ;
typedef void (*OPERFUNC) (struct tag_operate *, int , int  );

typedef struct tag_operate {
    int     opname;
    int     backup;
    int     oper_step;
    void    (*oper_init)   ( struct tag_operate *);
    void    (*oper_finish) ( struct tag_operate *);
    OPERFUNC onlbuttondown;
    OPERFUNC onlbuttonup;
    OPERFUNC onrbutton;
    OPERFUNC onmousemove;
    void     (*modechange) ( struct tag_operate *, int mode);
    UINT    add;
    int     x, y;
} OPERATIONINFO, *POPERATIONINFO;

                              

void pointInit (POPERATIONINFO this);
void pointFinish (POPERATIONINFO this) ;
void pointLButtonDown (POPERATIONINFO this, int x, int y ) ;
void pointLButtonUp (POPERATIONINFO this, int x, int y ) ;
void pointRButton (POPERATIONINFO this, int x, int y ) ;
void pointMouseMove (POPERATIONINFO this, int x, int y) ;

void lineInit (POPERATIONINFO this);
void lineFinish (POPERATIONINFO this) ;
void lineLButtonDown (POPERATIONINFO this, int x, int y ) ;
void lineLButtonUp (POPERATIONINFO this, int x, int y ) ;
void lineRButton (POPERATIONINFO this, int x, int y ) ;
void lineMouseMove (POPERATIONINFO this, int x, int y) ;
void lineModeChange ( POPERATIONINFO this, int mode );

void rectInit (POPERATIONINFO this);
void rectFinish (POPERATIONINFO this) ;
void rectLButtonDown (POPERATIONINFO this, int x, int y ) ;
void rectLButtonUp (POPERATIONINFO this, int x, int y ) ;
void rectRButton (POPERATIONINFO this, int x, int y ) ;
void rectMouseMove (POPERATIONINFO this, int x, int y) ;
void rectModeChange ( POPERATIONINFO this, int mode );

void ellipInit (POPERATIONINFO this);
void ellipFinish (POPERATIONINFO this) ;
void ellipLButtonDown (POPERATIONINFO this, int x, int y ) ;
void ellipLButtonUp (POPERATIONINFO this, int x, int y ) ;
void ellipRButton (POPERATIONINFO this, int x, int y ) ;
void ellipMouseMove (POPERATIONINFO this, int x, int y) ;
void ellipModeChange ( POPERATIONINFO this, int mode );

void rndrectInit (POPERATIONINFO this);
void rndrectFinish (POPERATIONINFO this) ;
void rndrectLButtonDown (POPERATIONINFO this, int x, int y ) ;
void rndrectLButtonUp (POPERATIONINFO this, int x, int y ) ;
void rndrectRButton (POPERATIONINFO this, int x, int y ) ;
void rndrectMouseMove (POPERATIONINFO this, int x, int y) ;
void rndrectModeChange ( POPERATIONINFO this, int mode );

void polyInit (POPERATIONINFO this);
void polyFinish (POPERATIONINFO this) ;
void polyLButtonDown (POPERATIONINFO this, int x, int y ) ;
void polyLButtonUp (POPERATIONINFO this, int x, int y ) ;
void polyRButton (POPERATIONINFO this, int x, int y ) ;
void polyMouseMove (POPERATIONINFO this, int x, int y) ;
void polyModeChange ( POPERATIONINFO this, int mode );

void getclrInit (POPERATIONINFO this);
void getclrFinish (POPERATIONINFO this) ;
void getclrLButtonDown (POPERATIONINFO this, int x, int y ) ;
void getclrLButtonUp (POPERATIONINFO this, int x, int y ) ;
void getclrRButton (POPERATIONINFO this, int x, int y ) ;
void getclrMouseMove (POPERATIONINFO this, int x, int y) ;
void getclrModeChange ( POPERATIONINFO this, int mode );

void eraseInit (POPERATIONINFO this);
void eraseFinish (POPERATIONINFO this) ;
void eraseLButtonDown (POPERATIONINFO this, int x, int y ) ;
void eraseLButtonUp (POPERATIONINFO this, int x, int y ) ;
void eraseRButton (POPERATIONINFO this, int x, int y ) ;
void eraseMouseMove (POPERATIONINFO this, int x, int y) ;
void eraseModeChange ( POPERATIONINFO this, int mode );

void fillInit (POPERATIONINFO this);
void fillFinish (POPERATIONINFO this) ;
void fillLButtonDown (POPERATIONINFO this, int x, int y ) ;
void fillLButtonUp (POPERATIONINFO this, int x, int y ) ;
void fillRButton (POPERATIONINFO this, int x, int y ) ;
void fillMouseMove (POPERATIONINFO this, int x, int y) ;
void fillModeChange ( POPERATIONINFO this, int mode );

void curveInit (POPERATIONINFO this);
void curveFinish (POPERATIONINFO this) ;
void curveLButtonDown (POPERATIONINFO this, int x, int y ) ;
void curveLButtonUp (POPERATIONINFO this, int x, int y ) ;
void curveRButton (POPERATIONINFO this, int x, int y ) ;
void curveMouseMove (POPERATIONINFO this, int x, int y) ;
void curveModeChange ( POPERATIONINFO this, int mode );

void wordInit (POPERATIONINFO this);
void wordFinish (POPERATIONINFO this) ;
void wordLButtonDown (POPERATIONINFO this, int x, int y ) ;
void wordLButtonUp (POPERATIONINFO this, int x, int y ) ;
void wordRButton (POPERATIONINFO this, int x, int y ) ;
void wordMouseMove (POPERATIONINFO this, int x, int y) ;
void wordModeChange ( POPERATIONINFO this, int mode );

void scalInit (POPERATIONINFO this);
void scalFinish (POPERATIONINFO this) ;
void scalLButtonDown (POPERATIONINFO this, int x, int y ) ;
void scalLButtonUp (POPERATIONINFO this, int x, int y ) ;
void scalRButton (POPERATIONINFO this, int x, int y ) ;
void scalMouseMove (POPERATIONINFO this, int x, int y) ;
void scalModeChange ( POPERATIONINFO this, int mode );

void sprayInit (POPERATIONINFO this);
void sprayFinish (POPERATIONINFO this) ;
void sprayLButtonDown (POPERATIONINFO this, int x, int y ) ;
void sprayLButtonUp (POPERATIONINFO this, int x, int y ) ;
void sprayRButton (POPERATIONINFO this, int x, int y ) ;
void sprayMouseMove (POPERATIONINFO this, int x, int y) ;
void sprayModeChange ( POPERATIONINFO this, int mode );

void advlnInit (POPERATIONINFO this);
void advlnFinish (POPERATIONINFO this) ;
void advlnLButtonDown (POPERATIONINFO this, int x, int y ) ;
void advlnLButtonUp (POPERATIONINFO this, int x, int y ) ;
void advlnRButton (POPERATIONINFO this, int x, int y ) ;
void advlnMouseMove (POPERATIONINFO this, int x, int y) ;
void advlnModeChange ( POPERATIONINFO this, int mode );

void selrInit (POPERATIONINFO this);
void selrFinish (POPERATIONINFO this) ;
void selrLButtonDown (POPERATIONINFO this, int x, int y ) ;
void selrLButtonUp (POPERATIONINFO this, int x, int y ) ;
void selrRButton (POPERATIONINFO this, int x, int y ) ;
void selrMouseMove (POPERATIONINFO this, int x, int y) ;
void selrModeChange ( POPERATIONINFO this, int mode );

void selregInit (POPERATIONINFO this);
void selregFinish (POPERATIONINFO this) ;
void selregLButtonDown (POPERATIONINFO this, int x, int y ) ;
void selregLButtonUp (POPERATIONINFO this, int x, int y ) ;
void selregRButton (POPERATIONINFO this, int x, int y ) ;
void selregMouseMove (POPERATIONINFO this, int x, int y) ;
void selregModeChange ( POPERATIONINFO this, int mode );

#endif
