/*    
** mgpaint: the mgpaint for MiniGUI.
** Copyright (C) 2002~2007  Feynman Software

** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License along
** with this program; if not, write to the Free Software Foundation, Inc.,
** 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/


#ifndef _FHTB_H
#define _FHTB_H


/**
 * \def CTRL_FHTOOLBAR
 * \brief The class name of the Fhas toolbar control.
 *
 * This control can be used for menu bar or caption bar.
 */
#define CTRL_FHTOOLBAR     "fhtoolbar"

#if 0
/* Structure of the toolbar control information */
typedef struct _FTINFO
{
    /* Image for displaying tool bar background. */
    PBITMAP bmp_bg;
} FTINFO;
#endif

/**
 * \def FT_TEXT_LEN
 * \brief Text length of the text field in the \a FTITEMINFO structure.
 */
#define FT_TEXT_LEN           32

/**
 * \def FT_TIP_LEN
 * \brief Text length of the tip field in the \a FTITEMINFO structure.
 */
#define FT_TIP_LEN            32


/**
 * \def FTIF_PUSHBUTTON
 * \brief Push button item type of a fhas toolbar item.
 *
 * Specified in the \a flags field in the \a FTITEMINFO structure.
 * An item of push button type acts like a real push button.
 */
#define FTIF_PUSHBUTTON   0x0001

/**
 * \def FTIF_CHKBUTTON
 * \brief Check button item type of a fhas toolbar item.
 *
 * Specified in the \a flags field in the \a FTITEMINFO structure.
 * An item of check button type acts like a real check button.
 */
#define FTIF_CHKBUTTON    0x0002

/**
 * \def FTIF_CHKBOX
 * \brief Check box item type of a fhas toolbar item.
 *
 * Specified in the \a flags field in the \a FTITEMINFO structure.
 * An item of check box type acts like a real check box.
 */
#define FTIF_CHKBOX       0x0003

/**
 * \def FTIF_MENUBUTTON
 * \brief menu button item type of a fhas toolbar item.
 *
 * Specified in the \a flags field in the \a FTITEMINFO structure.
 * An item of menu button item type acts like a real menu button.
 */
#define FTIF_MENUBUTTON   0x0004

/**
 * \def FTIF_STATIC
 * \brief static item type of a fhas toolbar item.
 *
 * Specified in the \a flags field in the \a FTITEMINFO structure.
 * An item of static type acts like a static label.
 */
#define FTIF_STATIC       0x0005

/**
 * \def FTIF_PANEL
 * \brief Panel type of a fhas toolbar item.
 *
 * Specified in the \a flags field in the \a FTITEMINFO structure.
 * An item of panel type is often used for a navigation panel.
 */
#define FTIF_PANEL        0x0006

/**
 * \def FTIF_BTMASK
 * \brief Bit mask of item type in the \a flag field.
 */
#define FTIF_BTMASK       0x000F

/**
 * \def FTIF_NORMAL
 * \brief Specifies the normal status of an item.
 *
 * Specified in the \a flags field in the \a FTITEMINFO structure.
 * If not specified, an item is in the normal status.
 */
#define FTIF_NORMAL       0x0000

/**
 * \def FTIF_CLICKED
 * \brief Specifies the clicked status of an item.
 *
 * Specified in the \a flags field in the \a FTITEMINFO structure.
 * Usable for an item with type FTIF_CHKBOX or FTIF_CHKBUTTON.
 */
#define FTIF_CLICKED      0x0010

/**
 * \def FTIF_HILIGHT
 * \brief Specifies the hilighted status of an item.
 *
 * Specified in the \a flags field in the \a FTITEMINFO structure.
 * Not used now.
 */
#define FTIF_HILIGHT      0x0020

/**
 * \def FTIF_DISABLED
 * \brief Specifies the disabled status of an item.
 *
 * Specified in the \a flags field in the \a FTITEMINFO structure.
 * If an item is disabled, it will not react to user operations, but
 * it is still visible on the tool bar.
 */
#define FTIF_DISABLED     0x0030

/**
 * \def FTIF_HIDE
 * \brief Specifies the hiden status of an item.
 *
 * Specified in the \a flags field in the \a FTITEMINFO structure.
 * If an item is hiden, it will not be shown on the tool bar.
 */
#define FTIF_HIDE         0x0040

/**
 * \def FTIF_STMASK
 * \brief Bit mask of item status in the \a flag field.
 */
#define FTIF_STMASK       0x00F0

/**
 * \def FTIF_TEXT
 * \brief The content type of the item is text.
 */
#define FTIF_TEXT         0x0100

/**
 * \def FTIF_BMP
 * \brief The content type of the item is bitmap.
 */
#define FTIF_BMP          0x0200

/**
 * \def FTIF_OWNERDRAW
 * \brief The content type of the item is drawn by the user.
 *
 * If an item is of this type, the user can sets the callback
 * draw function of an item through FTM_SETITEMOWNERDRAW message.
 */
#define FTIF_OWNERDRAW    0x0400
/*-----------------------------------------------------------*/
/**
 * \def FTIF_CENTERALIGN
 * \brief The content type of the item is center-align.
 */
#define FTIF_CENTERALIGN  0x00010000

/**
 * \def FTIF_RIGHTALIGN
 * \brief The content type of the item is right-align.
 */
#define FTIF_RIGHTALIGN   0x00020000

/**
 * \def FTIF_MASK
 * \brief Bit mask of item flags in the \a flags field.
 */
#define FTIF_ALIGNMASK     0x000F0000

/*-----------------------------------------------------------*/

/**
 * \def FTIF_BMP_2
 * \brief Used to specify the sub bitmap number as 2 in a bitmap of an
 *        item with FTIF_BMP content type.
 */
#define FTIF_BMP_2        0x1000

/**
 * \def FTIF_BMP_3
 * \brief Used to specify the sub bitmap number as 3 in a bitmap of an
 *        item with FTIF_BMP content type.
 */
#define FTIF_BMP_3        0x2000

/**
 * \def FTIF_BMP_4
 * \brief Used to specify the sub bitmap number as 4 in a bitmap of an
 *        item with FTIF_BMP content type.
 */
#define FTIF_BMP_4        0x3000

/**
 * \def FTIF_BMPNRMASK
 * \brief Bit mask of the sub bitmap number in the \a flag field.
 */
#define FTIF_BMPNRMASK    0x3000


/* action type, Internal used */
#define FTIF_UPACT        0x4000
/* action type, Internal used */
#define FTIF_DOWNACT      0x8000
/* action type, Internal used */
#define FTIF_UPDOWNMASK   0xC000


/**
 * \def FT_WHICH_FLAGS
 * \brief Sets the flags of an item.
 *
 * Specified in the \a which field in the \a FTITEMINFO structure.
 */
#define FT_WHICH_FLAGS    0x0001

/**
 * \def FT_WHICH_ID
 * \brief Sets the id of an item.
 *
 * Specified in the \a which field in the \a FTITEMINFO structure.
 */
#define FT_WHICH_ID       0x0002

/**
 * \def FT_WHICH_TEXT
 * \brief Sets the text of an item.
 *
 * Specified in the \a which field in the \a FTITEMINFO structure.
 */
#define FT_WHICH_TEXT     0x0004

/**
 * \def FT_WHICH_BMP
 * \brief Sets the bitmap of an item.
 *
 * Specified in the \a which field in the \a FTITEMINFO structure.
 */
#define FT_WHICH_BMP      0x0008

/**
 * \def FT_WHICH_TIP
 * \brief Sets the tip of an item.
 *
 * Specified in the \a which field in the \a FTITEMINFO structure.
 */
#define FT_WHICH_TIP      0x0010

/**
 * \def FT_WHICH_ADDDATA
 * \brief Sets the additional data of an item.
 *
 * Specified in the \a which field in the \a FTITEMINFO structure.
 */
#define FT_WHICH_ADDDATA  0x0020

/**
 * \def FT_WHICH_TYPEDATA
 * \brief Sets the type data of an item.
 *
 * Specified in the \a which field in the \a FTITEMINFO structure.
 */
#define FT_WHICH_TYPEDATA 0x0040


/**
 * \def FTS_TRANSPARENTBK
 * \brief The tool bar has a transparent background window style.
 */
#define FTS_TRANSPARENTBK 0x000000001L

/**
 * \def FTS_BMPBK
 * \brief The tool bar has a bitmap background window style.
 */
#define FTS_BMPBK         0x000000002L

/**
 * \def FTS_RIGHTALIGN
 * \brief The tool bar has a right-aligned window style.
 *
 * In the normal case, the tool bar is left-aligned, means that
 * the items is aligned from left to right.
 */
#define FTS_RIGHTALIGN    0x000000004L

/**
 * \def FTS_AUTORESIZE
 * \brief The tool bar has a auto resize window style.
 *
 * If a tool bar is auto resized, the size of it will be decided
 * according to the sizes of its items.
 */
#define FTS_AUTORESIZE    0x000000008L


/** 
 * Structure used to describe a toolbar item.
 */
typedef struct _FTITEMINFO
{
    /** 
     * Used by the \a FTM_SETITEM message to specify the operations,
     * can be combination of the following values:
     * - FT_WHICH_FLAGS\n
     * - FT_WHICH_ID\n
     * - FT_WHICH_TEXT\n
     * - FT_WHICH_BMP\n
     * - FT_WHICH_TIP\n
     * - FT_WHICH_ADDDATA\n
     * - FT_WHICH_TYPEDATA\n
     */
    DWORD       which;

    /**
     * Item style and type flags,
     * can be combination of the following values:
     * - FTIF_PUSHBUTTON\n
     * - FTIF_CHKBUTTON\n
     * - FTIF_CHKBOX\n
     * - FTIF_MENUBUTTON\n
     * - FTIF_STATIC\n
     * - FTIF_PANEL\n
     * - FTIF_NORMAL\n
     * - FTIF_CLICKED\n
     * - FTIF_HILIGHT\n
     * - FTIF_DISABLED\n
     * - FTIF_HIDE\n
     * - FTIF_TEXT\n
     * - FTIF_BMP\n
     * - FTIF_LEFT\n
     * - FTIF_RIGHTALIGN\n
     * - FTIF_CENTERALIGN\n
     * - FTIF_OWNERDRAW\n
     * - FTIF_BMP_2\n
     * - FTIF_BMP_3\n
     * - FTIF_BMP_4\n
     */
    DWORD       flags;

    /** item id */
    int         id;

    /** item text */
    const char* text;

    /** item tip text */
    const char* tip;

    /** item icon bitmap */
    PBITMAP     bmp;

    /** text color of item */
    int         text_color;

    /** item width */
    int         w;

    /** item additional data */
    DWORD       add_data;

    /** additional data related to the item type */
    void*       type_data;
} FTITEMINFO;
typedef FTITEMINFO* PFTITEMINFO;

/**
 * \var typedef void (*FTITEM_DRAW) (HDC hdc, RECT *rcItem, int id)
 * \brief Type of draw-item callback function.
 *
 * \param hdc    The drawing context.
 * \param rcItem The drawing range.
 * \param id     ID of the item.
 */
typedef void (*FTITEM_DRAW) (HDC hdc, RECT *rcItem, int id);


/**
 * \def FT_OKAY
 * \brief Means the operation is successfull.
 */
#define FT_OKAY        0
/**
 * \def FT_ERR
 * \brief Means the operation goes wrong.
 */
#define FT_ERR         (-1)


/**
 * \def FTM_ADDITEM
 * \brief Adds an item to a fhas tool bar.
 *
 * \code
 * FTM_ADDITEM
 * int index;
 * FTITEMINFO *fbi;
 * 
 * wParam = index;
 * lParam = (LPARAM)fbi;
 * \endcode
 *
 * \param index The desired position index of the new item in the tool bar.
 * \param fbi   Pointer to an initialized FTITEMINFO structure storing the 
 *              item info.
 *
 * \return The actual position index of the newly added item on success, 
 *         -1 if fails.
 */
#define FTM_ADDITEM            (MSG_USER+100)

/**
 * \def FTM_DELITEM
 * \brief Deletes an item in a fhas tool bar.
 *
 * \code
 * FTM_DELITEM
 * int id;
 *
 * wParam = id;
 * lParam = 0;
 * \endcode
 *
 * \param id ID of the item to be deleted.
 * \return FT_OKAY on success, FT_ERR if fails.
 */
#define FTM_DELITEM            (MSG_USER+101)

/* not implemented */
#define FTM_GETITEM            (MSG_USER+102)

/**
 * \def FTM_SETITEM
 * \brief Sets the information and properties of an item in a tool bar.
 *
 * \code
 * FTM_SETITEM
 * int id;
 * FTITEMINFO *fbi;
 *
 * wParam = id;
 * lParam = (LPARAM)fbi;
 * \endcode
 *
 * \param id  ID of the item to be set.
 * \param fbi Pointer to an initialized FTITEMINFO structure storing the 
 *            new item information.
 *
 * \return FT_OKAY on success, FT_ERR if fails.
 */
#define FTM_SETITEM            (MSG_USER+103)

/**
 * \def FTM_ENABLEITEM
 * \brief Enables an item in a tool bar.
 *
 * \code
 * FTM_ENABLEITEM
 * int id;
 *
 * wParam = id;
 * lParam = 0;
 * \endcode
 *
 * \param id  ID of the item to be enabled.
 * \return FT_OKAY on success, FT_ERR if fails.
 */
#define FTM_ENABLEITEM         (MSG_USER+104)

/**
 * \def FTM_DISABLEITEM
 * \brief Disables an item in a tool bar.
 *
 * \code
 * FTM_DISABLEITEM
 * int id;
 * BOOL Hide;
 *
 * wParam = id;
 * lParam = (LPARAM)Hide;
 * \endcode
 *
 * \param id   ID of the item to be disabled.
 * \param Hide If TRUE, hides the item when disabled.
 *
 * \return FT_OKAY on success, FT_ERR if fails.
 */
#define FTM_DISABLEITEM        (MSG_USER+105)

/**
 * \def FTM_SHOWITEM
 * \brief Shows or hides an item.
 *
 * \code
 * FTM_SHOWITEM
 * int id;
 * BOOL bShow;
 *
 * wParam = id;
 * lParam = (LPARAM)bShow;
 * \endcode
 *
 * \param id    ID of the item.
 * \param bShow If TRUE, Shows the item.
 *
 * \return FT_OKAY on success, FT_ERR if fails.
 */
#define FTM_SHOWITEM           (MSG_USER+106)

/**
 * \def FTM_DISABLEALL
 * \brief Disables the whole tool bar items.
 *
 * \code
 * FTM_DISABLEALL
 *
 * wParam = 0;
 * lParam = 0;
 * \endcode
 *
 * \return FT_OKAY on success, FT_ERR if fails.
 */
#define FTM_DISABLEALL         (MSG_USER+107)

/**
 * \def FTM_SETITEMTEXT
 * \brief Sets text of an item.
 *
 * \code
 * FTM_SETITEMTEXT
 * int id;
 * const char *text;
 *
 * wParam = id;
 * lParam = (LPARAM)text;
 * \endcode
 *
 * \param id   ID of the item to set text.
 * \param text New text content of the item.
 *
 * \return FT_OKAY on success, FT_ERR if fails.
 */
#define FTM_SETITEMTEXT        (MSG_USER+108)

/**
 * \def FTM_SETCHK
 * \brief Sets check status of an item.
 *
 * \code
 * FTM_SETCHK
 * int id;
 * BOOL bClicked;
 *
 * wParam = id;
 * lParam = bClicked;
 * \endcode
 *
 * \param id       ID of the item.
 * \param bClicked If TRUE, sets the item as checked status, or else as
 *                 normal status.
 *
 * \return FT_OKAY on success, FT_ERR if fails.
 */
#define FTM_SETCHK             (MSG_USER+109)

/**
 * \def FTM_SETITEMBMP
 * \brief Sets the bitmap of an item.
 *
 * \code
 * FTM_SETITEMBMP
 * int id;
 * PBITMAP pbmp;
 *
 * wParam = id;
 * lParam = (LPARAM)pbmp;
 * \endcode
 *
 * \param id   ID of the item.
 * \param pbmp Pointer to a BITMAP.
 *
 * \return FT_OKAY on success, FT_ERR if fails.
 */
#define FTM_SETITEMBMP         (MSG_USER+110)

/**
 * \def FTM_SETITEMOWNERDRAW
 * \brief Sets the owner draw function of an item with type FTIF_OWNERDRAW.
 *
 * \code
 * FTM_SETITEMOWNERDRAW
 * int id;
 * FTITEM_DRAW pfnDrawItem;
 *
 * wParam = id;
 * lParam = (LPARAM)pfnDrawItem;
 * \endocde
 *
 * \param id          ID of the item.
 * \param pfnDrawItem the new draw function of the item.
 *
 * \return FT_OKAY on success, FT_ERR if fails.
 */
#define FTM_SETITEMOWNERDRAW   (MSG_USER+111)

/**
 * \def FTM_UPDATEITEM
 * \brief Updates the content of an item.
 *
 * \code
 * int id;
 *
 * wParam = id;
 * lParam = 0;
 * \endocde
 *
 * \param id ID of the item.
 *
 * \return FT_OKAY on success, FT_ERR if fails.
 */
#define FTM_UPDATEITEM         (MSG_USER+112)

/**
 * \def FTM_RESETCONTENT
 * \brief Resets the whole tool bar.
 */
#define FTM_RESETCONTENT       (MSG_USER+113)

/**
 * \def FTM_GETITEMADDDATA
 * \brief Gets additional data of an item.
 *
 * \code
 * int id;
 *
 * wParam = id;
 * lParam = 0;
 * \endcode
 *
 * \param id ID of the item.
 * \return Additional data value of the item.
 */
#define FTM_GETITEMADDDATA     (MSG_USER+114)

/**
 * \def FTM_SETITEMADDDATA
 * \brief Sets the additional data of an item.
 *
 * \code
 * int id;
 * DWORD newdata;
 *
 * wParam = id;
 * lParam = newdata;
 * \endcode
 *
 * \param id      ID of the item.
 * \param newdata The new additional data of the item.
 *
 * \return The old additional data value of the item.
 */
#define FTM_SETITEMADDDATA     (MSG_USER+115)

/**
 * \def FTM_GETITEMBYDATA
 * \brief Gets an tool bar item by the additional data value.
 *
 * \code
 * FTM_GETITEMBYDATA
 * DWORD addData;
 *
 * wParam = addData;
 * lParam = 0;
 * \endcode
 *
 * \param addData Additional data of the target item.
 *
 * \return ID of the target item on success, FT_ERR if fails.
 */
#define FTM_GETITEMBYDATA      (MSG_USER+116)

/**
 * \def FTM_GETITEMBYIDX
 * \brief Gets an tool bar item by its position index.
 *
 * \code
 * FTM_GETITEMBYIDX
 * int index;
 *
 * wParam = index;
 * lParam = 0;
 * \endcode
 *
 * \param index Position index of the item.
 *
 * \return ID of the target item on success, FT_ERR if fails.
 */
#define FTM_GETITEMBYIDX       (MSG_USER+117)

/**
 * \def FTM_SETPANELITEM_STATUS
 * \brief Sets the status of a panel item.
 *
 * \code
 * FTM_SETPANELITEM_STATUS
 * int id;
 * int status;
 *
 * wParam = id;
 * lParam = status;
 * \endcode
 *
 * \param id     ID of the item.
 * \param status The new status of the item.
 *
 * \return FT_OKAY on success, FT_ERR if fails.
 */
#define FTM_SETPANELITEM_STATUS (MSG_USER+118)

/**
 * \def FTM_GETPANELITEM_STATUS
 * \brief Gets status of a panel item.
 *
 * \code
 * FTM_GETPANELITEM_STATUS
 * int id;
 *
 * wParam = id;
 * lParam = 0;
 * \endcode
 *
 * \param id ID of the item.
 *
 * \return Status value of the panel item on success, 0 if fails.
 */
#define FTM_GETPANELITEM_STATUS (MSG_USER+119)


#endif /* _FHTB_H */
